#include <stdlib.h>
#include <sys/time.h>
#include <iostream>
#include <algorithm>

#include "bitonic.h"
#include "meta_copy.h"

// Define the type to be sorted (compiler switch -DSORT_TYPE=X overrides)
#ifndef SORT_TYPE
#define SORT_TYPE int
#endif

// Define the list size (compiler switch -DSORT_SIZE=# overrids)
#ifndef SORT_SIZE
#define SORT_SIZE 16
#endif

typedef SORT_TYPE sort_type ;

double difftime(timeval t1, timeval t2)
{
  double dt1 = t1.tv_sec + t1.tv_usec*1e-6 ;
  double dt2 = t2.tv_sec + t2.tv_usec*1e-6 ;
  return dt2-dt1 ;
}



int main(int ac,char **av[]) {
  const int N=SORT_SIZE ;

  // To measure the times of very small increments, repeat operation many
  // times until the overall time is about 1 sec.  Then divide by the number
  // of times the operation was performed in order to get an estimate of
  // the time required to perform one iteration
  const int SAMPLE = 100000 ;

  // Allocate space for the sorting test
  sort_type *test = new sort_type[N], *scratch = new sort_type[N] ;

  // Create a random sequence to sort
  for(int i=0;i<N;++i) 
    scratch[i] = sort_type(drand48()*double(1<<31)) ;
  

  // We will store the start and end times using these data structures
  timeval t1, t2 ;


  // Get the start time
  gettimeofday(&t1,NULL) ;

  // Loop SAMPLE times and repeat operation.  Here we are measureing the
  // copy time plus sort time
  for(int i=0;i<SAMPLE;++i) {
    meta_copy<sort_type,N>::copy(test,scratch) ;
    bitonic_sort<sort_type,N>::sort(test) ;
  }

  // Get the ending time
  gettimeofday(&t2,NULL) ;

  // difftime lets us compute the time in seconds between two timevals.
  // We convert to the time per loop iteration in microseconds.
  cout << "time for bitonic sort is " << difftime(t1,t2)/double(SAMPLE)*1e6
       << endl;

}

